#include <QLabel>
#include <QPushButton>
#include <QSlider>
#include <QScrollArea>
#include <QString>
#include <QImage>
#include <QMessageBox>
#include <QSpinBox>

#include "ARsql.h"

#include "ARDialogAnteprima.h"

ARDialogAnteprima::ARDialogAnteprima(QWidget *parent) : QDialog(parent)
 {
 	act_pag=1; zo=1.0;
 	sql = new ARsql(this);
  pic = new QImage(0,0,QImage::Format_RGB32);

  warnbox = new QMessageBox(tr("Avviso"),"",QMessageBox::Warning,0,0,0,this);
  warnbox->setWindowModality(Qt::WindowModal);

  _act_pag = new QLabel("",this); _act_pag->setGeometry(618,204,50,24);
  _zoom  = new QLabel("Zoom:",this);  _zoom->setGeometry(535,102,50,24);
  _pag   = new QLabel("Pagine:",this); _pag->setGeometry(535,182,50,24);
  _ini   = new QLabel("Stampa la prima pagina da:",this); _ini->setGeometry(535,234,200,24);
  _pic   = new QLabel("",this);
  _pic->setBackgroundRole(QPalette::Dark);
  _pic->setAlignment(Qt::AlignTop); 
  area   = new QScrollArea(this); area->setGeometry(2,3,529,557);
  area->setWidget(_pic);
  zoom   = new QSlider(Qt::Horizontal,this);
  zoom->setGeometry(534,160,140,20);
  zoom->setRange(1,200); zoom->setValue(100);
 	zoomp  = new QPushButton(QIcon("pic/zoom+.png"),"",this);
  zoomp->setGeometry(574,124,32,32);  zoomp->setIconSize(QSize(28,28));
 	zoomm  = new QPushButton(QIcon("pic/zoom-.png"),"",this);
  zoomm->setGeometry(534,124,32,32);  zoomm->setIconSize(QSize(28,28));
 	zoomc  = new QPushButton(QIcon("pic/zoom100.png"),"",this);
 	zoomc->setGeometry(614,124,60,32);  zoomc->setIconSize(QSize(28,28));
  prima  = new QPushButton("|<",this); prima->setGeometry (535,204,24,24);
  prec   = new QPushButton("<",this);  prec->setGeometry  (560,204,24,24);
  succ   = new QPushButton(">",this);  succ->setGeometry  (586,204,24,24);
  _ini_c = new QLabel("Colonna",this); _ini_c->setGeometry(535,260,60,24);
  ini_c  = new QSpinBox(this);         ini_c->setGeometry (590,260,40,24);
  ini_c->setRange(1,1); ini_c->setValue(1);
  _ini_r = new QLabel("Riga",this);    _ini_r->setGeometry(535,290,60,24);
  ini_r  = new QSpinBox(this);         ini_r->setGeometry (590,290,40,24);
  ini_r->setRange(1,1); ini_r->setValue(1);

 	setstampa = new QPushButton(QIcon("pic/printer_setup.png"),tr(" Imposta stampante"),this);
  setstampa->setGeometry(534,2,140,50); setstampa->setIconSize(QSize(32,32));
 	prefstampa = new QPushButton(QIcon("pic/pref_printer.png"),tr(" Imposta stampe"),this);
  prefstampa->setGeometry(534,52,140,50); prefstampa->setIconSize(QSize(32,32));
 	stampa = new QPushButton(QIcon("pic/printer.png"),tr(" Avvia stampa"),this);
  stampa->setGeometry(534,462,140,50); stampa->setIconSize(QSize(32,32));
 	annulla = new QPushButton(QIcon("pic/exit.png"),tr(" Annulla stampa"),this);
  annulla->setGeometry(534,511,140,50); annulla->setIconSize(QSize(32,32));

  connect(annulla,SIGNAL(clicked()),this,SLOT(reject()));
  connect(stampa,SIGNAL(clicked()),this,SLOT(accept()));
  connect(zoomp,SIGNAL(clicked()),this,SLOT(zoomPiu()));
  connect(zoomm,SIGNAL(clicked()),this,SLOT(zoomMeno()));
  connect(zoomc,SIGNAL(clicked()),this,SLOT(zoom100()));
  connect(zoom,SIGNAL(valueChanged(int)),this,SLOT(zoomBarra(int)));

  setWindowFlags(Qt::Window | Qt::MSWindowsFixedSizeDialogHint | Qt::WindowTitleHint | Qt::WindowSystemMenuHint);
  setWindowTitle(tr("Anteprima di stampa"));
  setGeometry(80,20,675,561);
 }

void ARDialogAnteprima::setAnteprima(int w, int h)
 {
 	pic->~QImage();
  pic = new QImage(w,h,QImage::Format_RGB32);
  pic->fill(0xFFFFFF);
 }

void ARDialogAnteprima::zoomBarra(int v) { zo=(float)v/100.0; ridisegna(); }

void ARDialogAnteprima::zoom100() 
 {
 	zo=1.0;
 	zoom->setValue(100);
 	//ridisegna();
 }

void ARDialogAnteprima::zoomMeno() 
 {
	zo=zo-0.1;
 	if(zo<0) zo=0.01;
 	zoom->setValue((int)(zo*100));
 	ridisegna();
 }

void ARDialogAnteprima::zoomPiu()
 { 
  zo=zo+0.1;
  if(zo>3) zo=3;
  zoom->setValue((int)(zo*100));
 	ridisegna();
 }

void ARDialogAnteprima::ridisegna()
 {
 	QImage aux;
 	
 	aux=pic->scaled((int)(pic->width()*zo),(int)(pic->height()*zo),Qt::IgnoreAspectRatio,Qt::SmoothTransformation);
  _pic->setPixmap(QPixmap::fromImage(aux));
  _pic->setGeometry(0,0,aux.width()+2,aux.height()+2);
  
 	_act_pag->setText(QString().sprintf("Pg. <b>%d</b>",act_pag));
 }

int ARDialogAnteprima::getPagina() { return act_pag; }

void ARDialogAnteprima::setPagina(int n) { act_pag=n; }

void ARDialogAnteprima::addPagina(int np) { act_pag=act_pag+np; if(act_pag==0) act_pag=1; }

QString ARDialogAnteprima::quote(QString str, bool like)
 {
 	if(like)
 	 {
 	  str.replace('\\',"\\\\\\\\");
 	  str.replace('\'',"\\'");
 	 }
 	else
   {
 	  str.replace('\\',"\\\\");
 	  str.replace('\'',"\\'");
   }
 	return str;
 }
