#include <QPushButton>
#include <QLineEdit>
#include <QComboBox>
#include <QLabel>
#include <QString>
#include <QMessageBox>
#include <QRegExp>

#include "ARFinestra.h"
#include "ARDialogProv.h"
#include "ARDialogCAP.h"
#include "ARFornitore.h"
#include "ARsql.h"

#include "ARDialogIndirReso.h"

ARDialogIndirReso::ARDialogIndirReso(QWidget *parent) : QDialog(parent)
 {
  mainWin=(ARFinestra *)parent;
  sql = new ARsql(this);
  provbox = new ARDialogProv(this);
 	capbox  = new ARDialogCAP(this);
  setWindowTitle(tr("Aggiungi indirizzo per reso merce"));
  yesnobox = new QMessageBox(tr("Scegli"),"",QMessageBox::Question,QMessageBox::Yes,QMessageBox::No,QMessageBox::NoButton,this);
  infobox  = new QMessageBox(tr("Informazioni"),"",QMessageBox::Question,0,0,0,this);
  warnbox  = new QMessageBox(tr("Avviso"),"",QMessageBox::Warning,0,0,0,this);
  _nome  = new QLabel(tr("Nome"),this);      _nome->setGeometry (10,   10,  80, 24);
  nome   = new QLineEdit("",this);           nome->setGeometry  (80,   10, 460, 24);
  _indr  = new QLabel(tr("Indirizzo"),this); _indr->setGeometry (10,   40,  80, 24);
  indr   = new QLineEdit("",this);           indr->setGeometry  (80,   40, 460, 24);
  _cap   = new QLabel(tr("C.A.P."),this);    _cap->setGeometry  (10,   70,  80, 24);
  cap    = new QLineEdit("",this);           cap->setGeometry   (80,   70,  42, 24);
  cercap = new QPushButton(QIcon("pic/cerca.png"),"",this);
  cercap->setGeometry(126,70,24,24);
  _citta = new QLabel(tr("Citt"),this);     _citta->setGeometry(155,  70,  80, 24);
  citta  = new QLineEdit("",this);           citta->setGeometry (185,  70, 222, 24);
  _prov  = new QLabel(tr("Provincia"),this); _prov->setGeometry (415,  70,  80, 24);
  prov   = new QLineEdit("",this);           prov->setGeometry  (470,  70,  42, 24);
  cercapr = new QPushButton(QIcon("pic/cerca.png"),"",this);
  cercapr->setGeometry  (517,70,24,24);
  _nazio = new QLabel(tr("Nazione"),this);   _nazio->setGeometry( 10, 100,  80, 24);
  nazio  = new QLineEdit(tr("Italia"),this); nazio->setGeometry ( 80, 100, 460, 24);
  _tel   = new QLabel(tr("Telefono"),this);  _tel->setGeometry  ( 10, 130,  80, 24);
  tel    = new QLineEdit("",this);           tel->setGeometry   ( 80, 130, 120, 24);
  _fax   = new QLabel(tr("Fax"),this);       _fax->setGeometry  (210, 130,  80, 24);
  fax    = new QLineEdit("",this);           fax->setGeometry   (240, 130, 120, 24);
  _cell  = new QLabel(tr("Cellulare"),this); _cell->setGeometry (370, 130,  80, 24);
  cell   = new QLineEdit("",this);           cell->setGeometry  (420, 130, 120, 24);
  _email = new QLabel(tr("Email"),this);     _email->setGeometry( 10, 160,  80, 24);
  email  = new QLineEdit("",this);           email->setGeometry ( 80, 160, 460, 24);
  reg_email = new QRegExp("^[a-z0-9][_\\.a-z0-9-]+@([a-z0-9][0-9a-z-]+\\.)+([a-z]{2,4})$");

  nome->setMaxLength(255);
  indr->setMaxLength(255);
  cap->setMaxLength(5);
  citta->setMaxLength(255);
  prov->setMaxLength(2);
  nazio->setMaxLength(255);
  tel->setMaxLength(50);
  fax->setMaxLength(50);
  cell->setMaxLength(50);
  email->setMaxLength(255);

  esegui  = new QPushButton(QIcon("pic/fornitori_add.png"),tr("Aggiungi"),this); esegui->setGeometry(8,190,106,30);
  annulla = new QPushButton(QIcon("pic/exit.png"),tr("Annulla"),this); annulla->setGeometry(118,190,106,30);

  connect(annulla,SIGNAL(clicked()),this,SLOT(reject()));
  connect(esegui,SIGNAL(clicked()),this,SLOT(addIndirReso()));
  connect(cercapr,SIGNAL(clicked()),this,SLOT(cercaProvincia()));
  connect(cercap,SIGNAL(clicked()),this,SLOT(cercaCAP()));
   
  yesnobox->setWindowModality(Qt::WindowModal);
  yesnobox->setButtonText(QMessageBox::Yes,tr("Si"));
  yesnobox->setButtonText(QMessageBox::No,tr("No"));
  infobox->setWindowModality(Qt::WindowModal);
  warnbox->setWindowModality(Qt::WindowModal);
 	
  setWindowFlags(Qt::Window | Qt::MSWindowsFixedSizeDialogHint | Qt::WindowTitleHint | Qt::WindowSystemMenuHint);
  setGeometry(160,160,550,230);
  azione=ARDIALOGINDIRRESO_ADD;
 }

void ARDialogIndirReso::cercaProvincia()
 {
 	provbox->clearForm();
 	if(provbox->exec()==QDialog::Accepted) prov->setText(provbox->getProvincia());
 }

void ARDialogIndirReso::cercaCAP()
 {
 	capbox->visCAP();
 	if(capbox->exec()==QDialog::Accepted)
 	 {
 	 	cap->setText(capbox->getCAP());
 	 	citta->setText(capbox->getCitta());
 	  prov->setText(capbox->getProvincia());
   }
 }

void ARDialogIndirReso::addIndirReso()
 {
 	QString nome=quote(this->nome->text());
  QString indr=quote(this->indr->text());
  QString cap=quote(this->cap->text());
  QString citta=quote(this->citta->text());
  QString prov=quote(this->prov->text());
  QString nazio=quote(this->nazio->text());
  QString tel=quote(this->tel->text());
  QString fax=quote(this->fax->text());
  QString cell=quote(this->cell->text());
  QString email=quote(this->email->text());
  QString ts=sql->timeStamp();
  QString data=sql->data(ts);
  QString ora=sql->ora(ts);

	if(isAzione(ARDIALOGINDIRRESO_ADD))
 	 {
 	  if(!checkForm())
 	   {
 	    if(sql->doQuery("INSERT INTO indirizzi_reso VALUES('0','"+fornitoreID+"','"+nome+"','"+indr+"','"+cap+"','"+citta+"','"+prov+"','"+nazio+"','"+tel+"','"+fax+"','"+cell+"','"+email+"','"+data+"','"+ora+"','"+ts+"')"))
           {
            mainWin->prendiAddFor()->visIndirReso();
            infobox->setText(tr("Indirizzo per il reso inserito"));
            infobox->exec();
            clearForm();
            reject();
 	     }
 	   }
 	 }
 	if(isAzione(ARDIALOGINDIRRESO_EDIT))
 	 {
 	  if(!checkForm())
 	   {
 	    if(sql->doQuery("UPDATE indirizzi_reso SET nome='"+nome+"', indirizzo='"+indr+"', cap='"+cap+"', citta='"+citta+"', provincia='"+prov+"', nazione='"+nazio+"', tel='"+tel+"', fax='"+fax+"', cell='"+cell+"', email='"+email+"' WHERE indir_resoID='"+indir_resoID+"'"))
 	     {
            mainWin->prendiAddFor()->visIndirReso();
            warnbox->setText(tr("Indirizzo per il reso modificato e salvato!"));
            warnbox->exec();
            reject();
           }
         }
 	 }
 }

bool ARDialogIndirReso::setForm(QString indir_resoID)
 {
  MYSQL_RES *rs;
  MYSQL_ROW row;

  rs=sql->doQueryRS("SELECT * FROM indirizzi_reso WHERE indir_resoID ='"+indir_resoID +"'");
  if(rs)
   {
    if(mysql_num_rows(rs)!=1) { mysql_free_result(rs); return false; }
    row=mysql_fetch_row(rs);
    nome->setFocus();
    nome->setText(row[2]);
    indr->setText(row[3]);
    cap->setText(row[4]);
    citta->setText(row[5]);
    prov->setText(row[6]);
    nazio->setText(row[7]);
    tel->setText(row[8]);
    fax->setText(row[9]);
    cell->setText(row[10]);
    email->setText(row[11]);
    mysql_free_result(rs);
    return true;
   }
  return false;
 }

void ARDialogIndirReso::setFornitoreID(QString forID) { fornitoreID=forID; }

void ARDialogIndirReso::setIndirResoID(QString iresoID) { indir_resoID=iresoID; }

void ARDialogIndirReso::setAzione(int azio) { azione=azio; }

bool ARDialogIndirReso::isAzione(int azio) { if(azione==azio) return true; else return false; }

void ARDialogIndirReso::clearForm()
 {
  nome->setFocus();
  nome->clear();
  indr->clear();
  cap->clear();
  citta->clear();
  prov->clear();
  nazio->setText(tr("Italia"));
  tel->clear();
  fax->clear();
  cell->clear();
  email->clear();
 }

bool ARDialogIndirReso::checkForm()
 {
 	if(quote(this->indr->text())=="")
 	 {
 	  warnbox->setText(tr("Devi inserire l'indirizzo per il reso"));
 	  warnbox->exec();
 	  return true;
 	 }
 	if(quote(this->email->text())!="" && !reg_email->exactMatch(quote(this->email->text())))
 	 {
 	  warnbox->setText(tr("Formato email errato"));
 	  warnbox->exec();
 	  return true;
 	 }
 	return false;
 }

QString ARDialogIndirReso::quote(QString str, bool like)
 {
 	if(like)
 	 {
 	  str.replace('\\',"\\\\\\\\");
 	  str.replace('\'',"\\'");
 	 }
 	else
   {
 	  str.replace('\\',"\\\\");
 	  str.replace('\'',"\\'");
   }
 	return str;
 }
