#include <QTableWidget>
#include <QTableWidgetItem>
#include <QGroupBox>
#include <QPushButton>
#include <QLineEdit>
#include <QComboBox>
#include <QLabel>
#include <QMessageBox>
#include <QMenu>
#include <QAction>
#include <QContextMenuEvent>
#include <QDateEdit>
#include <QCheckBox>

#include "ARsql.h"

#include "ARGrigliaSca.h"

ARGrigliaSca::ARGrigliaSca(QWidget *parent) : QWidget(parent)
 {
 	num_col=8;
 	sql = new ARsql(this);
  infobox    = new QMessageBox(tr("Informazioni"),"",QMessageBox::Question,0,0,0,this);
  warnbox    = new QMessageBox(tr("Avviso"),"",QMessageBox::Warning,0,0,0,this);
  yesnobox   = new QMessageBox(tr("Scegli"),"",QMessageBox::Question,QMessageBox::Yes,QMessageBox::No,QMessageBox::NoButton,this);
 	cornice    = new QGroupBox(tr("Ricerca Scadenza "),this);

  _for       = new QLabel(tr("Fornitore"),cornice);       _for->setGeometry      (  6, 15, 80, 24);
  listaFor   = new QComboBox(cornice);                    listaFor->setGeometry  ( 60, 15,270, 24);
  _descr     = new QLabel(tr("Descrizione"),cornice);     _descr->setGeometry    (337, 15, 80, 24);
  descr      = new QLineEdit("",cornice);                 descr->setGeometry     (400, 15,295, 24);
  _somma     = new QLabel(tr("Somma"),cornice);           _somma->setGeometry    (  6, 45, 80, 24);
  somma      = new QLineEdit("",cornice);                 somma->setGeometry     ( 60, 45,150, 24);
  somma->setValidator(new QDoubleValidator(-50000000.0,50000000.0,2,somma));
  _note      = new QLabel(tr("Note"),cornice);            _note->setGeometry     (220, 45, 80, 24);
  note       = new QLineEdit("",cornice);                 note->setGeometry      (255, 45,440, 24);
  note->setMaxLength(255);
  _stato     = new QLabel(tr("Visualizza"),cornice);      _stato->setGeometry    (  6, 80, 80, 24);
  listaStato = new QComboBox(cornice);                    listaStato->setGeometry( 60, 80,115, 24);
  listaStato->addItem(tr("Tutte le scadenze"),QVariant(-1));
  listaStato->addItem(tr("Pagate"),QVariant(1));
  listaStato->addItem(tr("Non Pagate"),QVariant(0));
  QDate cd = QDate::currentDate();
  _di        = new QLabel(tr("Scadenze di"),cornice);     _di->setGeometry       (181, 80, 80, 24);
  di         = new QDateEdit(cd,cornice);                 di->setGeometry        (250, 80, 98, 24);
  di->setDisplayFormat("MMMM-yyyy");
  ckdata  = new QCheckBox("Disattiva ricerca per data",cornice);
 	ckdata->setGeometry(355,80,150,24);
  
 	edit      = new QPushButton(QIcon("pic/edit.png"),tr("Visualizza"),this);
 	del       = new QPushButton(QIcon("pic/delete.png"),tr("Cancella"),this);
  stampa    = new QPushButton(QIcon("pic/tabulati.png"),tr("Stampa"),this);
 	reset = new QPushButton(QIcon("pic/no_search.png"),tr("Azzera"),cornice);
  reset->setGeometry(503,83,96,24);
 	search = new QPushButton(QIcon("pic/search.png"),tr("Cerca"),cornice);
  search->setGeometry(601,83,96,24);
 	table = new QTableWidget(this);
  table->setSortingEnabled(true);
  table->setAlternatingRowColors(true);
  table->setEditTriggers(QAbstractItemView::NoEditTriggers);
  table->setSelectionMode(QAbstractItemView::SingleSelection);
  table->setSelectionBehavior(QAbstractItemView::SelectRows);
 	for(int i=0;i<num_col;i++) table->insertColumn(i);
 	table->setColumnWidth(0,20); table->hideColumn(0);
 	table->setColumnWidth(1,220);
 	table->setColumnWidth(2,220);
 	table->setColumnWidth(3,100);
 	table->setColumnWidth(4,150);
 	table->setColumnWidth(5,250);
 	table->setColumnWidth(6,10); table->hideColumn(6);
 	table->setColumnWidth(7,10); table->hideColumn(7);
 	table->setHorizontalHeaderLabels(QStringList() << "ID" << "Fornitore" << "Descrizione" << "Somma" << "Scade il" << "Note");

 	editCMnu=new QAction(QIcon("pic/edit.png"),tr("Visualizza la scadenza selezionata"),this);
 	delCMnu=new QAction(QIcon("pic/delete.png"),tr("Cancella la scadenza selezionata"),this);
 	pagaCMnu=new QAction(QIcon("pic/pay.png"),tr("Segna la scadenza come pagata"),this);
 	npagCMnu=new QAction(QIcon("pic/pay_no.png"),tr("Segna la scadenza come non pagata"),this);

 	connect(del,SIGNAL(clicked()),this,SLOT(delScadenze()));
 	connect(search,SIGNAL(clicked()),this,SLOT(visScadenze()));
 	connect(reset,SIGNAL(clicked()),this,SLOT(resRicerca()));
 	connect(listaFor,SIGNAL(activated(int)),this,SLOT(visScadenze(int)));
 	connect(listaStato,SIGNAL(activated(int)),this,SLOT(visScadenze(int)));
 	connect(descr,SIGNAL(returnPressed()),this,SLOT(visScadenze()));
  connect(note,SIGNAL(returnPressed()),this,SLOT(visScadenze()));
  connect(somma,SIGNAL(returnPressed()),this,SLOT(visScadenze()));
  connect(ckdata,SIGNAL(stateChanged(int)),this,SLOT(visScadenze(int)));
  connect(di,SIGNAL(dateChanged(QDate)),this,SLOT(visScadenze(QDate)));
  // Menu contestuale
  connect(pagaCMnu,SIGNAL(triggered()),this,SLOT(pagaScadenze()));
  connect(npagCMnu,SIGNAL(triggered()),this,SLOT(nonPagaScadenze()));
  connect(delCMnu,SIGNAL(triggered()),this,SLOT(delScadenze()));

 	infobox->setWindowModality(Qt::WindowModal);
 	warnbox->setWindowModality(Qt::WindowModal);
 	yesnobox->setWindowModality(Qt::WindowModal);
 	yesnobox->setButtonText(QMessageBox::Yes,tr("Si"));
 	yesnobox->setButtonText(QMessageBox::No,tr("No"));
  hide();
 }

void ARGrigliaSca::settaGeometria(int lar, int alt)
 {
  setGeometry(0,61,lar,alt-70);
 	table->setGeometry(0,0,lar,alt-220);
 	cornice->setGeometry(2,alt-222,700,110);
  edit->setGeometry   (704,alt-213,96,24);
  del->setGeometry    (704,alt-188,96,24);
  stampa->setGeometry (704,alt-163,96,48);
 }

void ARGrigliaSca::visScadenze(int) { visScadenze(); }

void ARGrigliaSca::visScadenze(QDate d) { visScadenze(); }

void ARGrigliaSca::visScadenze()
 {
 	MYSQL_RES *rs;
 	MYSQL_ROW row;
 	QString scadenzaID,fornitoreID;
 	QString query,note,somma,descr,data_scadenza,stato;
 	bool primo;
 	int r;
 	
 	populateFor(listaFor->itemData(listaFor->currentIndex()).toString());
 	table->setRowCount(0);
 	r=0; primo=true;
 	query ="SELECT scadenze.scadenzaID,scadenze.fornitoreID,scadenze.descrizione,scadenze.note,scadenze.somma,scadenze.stato,scadenze.data_scadenza,";
  query+="       fornitori.nome,fornitori.indirizzo,fornitori.cap,fornitori.citta,fornitori.provincia,fornitori.nazione ";
  query+="FROM   scadenze,fornitori";

 	fornitoreID = listaFor->itemData(listaFor->currentIndex()).toString();
 	stato       = listaStato->itemData(listaStato->currentIndex()).toString();
 	descr = quote(this->descr->text(),true);
 	note = quote(this->note->text(),true);
 	somma = quote(this->somma->text(),true);
 	if(fornitoreID!="0") { if(primo) { query+=" WHERE scadenze.fornitoreID='"+fornitoreID+"'"; primo=false; } else { query+=" AND scadenze.fornitoreID='"+fornitoreID+"'"; } }
 	if(stato!="-1")      { if(primo) { query+=" WHERE stato='"+stato+"'"; primo=false; } else { query+=" AND stato='"+stato+"'"; } }
 	if(descr!="")        { if(primo) { query+=" WHERE descrizione LIKE '%"+descr+"%'"; primo=false; } else { query+=" AND descrizione LIKE '%"+descr+"%'"; } }
 	if(note!="")         { if(primo) { query+=" WHERE note LIKE '%"+note+"%'"; primo=false; } else { query+=" AND note LIKE '%"+note+"%'"; } }
 	if(somma!="")        { if(primo) { query+=" WHERE somma LIKE '%"+somma+"%'"; primo=false; } else { query+=" AND somma LIKE '%"+somma+"%'"; } }
 	if(ckdata->checkState()==Qt::Unchecked)
 	 if(primo) { query+=" WHERE scadenze.data_scadenza LIKE '"+di->date().toString("yyyy-MM")+"%'"; primo=false; } else { query+=" AND scadenze.data_scadenza LIKE '"+di->date().toString("yyyy-MM")+"%'"; }
 	if(fornitoreID!="0") if(primo) { query+=" WHERE scadenze.fornitoreID=fornitori.fornitoreID"; primo=false; } else { query+=" AND scadenze.fornitoreID=fornitori.fornitoreID"; }
 	else query+=" GROUP BY scadenzaID,scadenze.fornitoreID";
 	query+=" ORDER BY scadenze.scadenzaID DESC";
 	rs=sql->doQueryRS(query);
 	
 	QString a_indr,a_cap,a_citta,a_prov,a_piva,a_cf;
 	if(rs)
 	 {
 	  while((row=mysql_fetch_row(rs)))
     {
   	  table->insertRow(r); table->setRowHeight(r,18);

      //a_indr=row[2]; a_cap=row[3]; a_citta=row[4]; a_prov=row[5];
   	  //a_indr=a_indr+"   "+a_cap+" "+a_citta;
   	  //if(a_prov!="") a_indr+=" ("+a_prov+")";

   	  table->setItem(r,0,inserisciRiga(row[0]));
   	  if(QString(row[1])=="0") table->setItem(r,1,inserisciRiga("---"));
   	  else
   	   {
   	   	if(fornitoreID!="0") table->setItem(r,1,inserisciRiga(row[7]));
   	   	else
   	   	 {
 	   	 	 	MYSQL_RES *rs1;
 	        MYSQL_ROW row1;

          rs1=sql->doQueryRS("SELECT * FROM fornitori WHERE fornitoreID='"+QString(row[1])+"'");
          if(mysql_num_rows(rs1)>0)
           {
            row1=mysql_fetch_row(rs1);
   	   	    table->setItem(r,1,inserisciRiga(row1[1]));
   	   	   }
   	   	  else table->setItem(r,1,inserisciRiga("???")); // Fornitore inesistente
   	   	  mysql_free_result(rs1);
   	   	 }
   	   }
   	  table->setItem(r,2,inserisciRiga(row[2]));        // Descrizione

   	  table->setItem(r,3,inserisciRiga(row[4]));        // Somma
   	  if(QString(row[5])=="0") table->item(r,3)->setBackgroundColor(QColor(255,140,140));
   	  if(QString(row[5])=="1") table->item(r,3)->setBackgroundColor(QColor(140,255,140));
   	  
   	  data_scadenza=row[6];
   	  QDate aux_date=QDate::fromString(data_scadenza,"yyyy-MM-dd");
   	  data_scadenza=aux_date.toString("dd-MMM-yyyy");
   	  table->setItem(r,4,inserisciRiga(data_scadenza)); // Scadenza
   	  QDate adesso = QDate::currentDate();
   	  // Il 15 in futuro dovr essere preso da database come opzione
   	  adesso=adesso.addDays(15);
   	  if(adesso>=aux_date) table->item(r,4)->setBackgroundColor(QColor(255,255,140));
   	  adesso=adesso.addDays(-15);
   	  if(adesso>=aux_date) table->item(r,4)->setBackgroundColor(QColor(255,140,140));
   	  
   	  table->setItem(r,5,inserisciRiga(row[3]));        // Note

   	  /*porto=row[5];
   	  if(porto=="1") porto=tr("Franco");
   	  table->setItem(r,6,inserisciRiga(porto));
   	  table->setItem(r,7,inserisciRiga(QString(row[2])+" "+row[3]+" "+row[4]));*/
   	  r++;
     }
    mysql_free_result(rs);
    if(r==0) 
     {
     	if(infobox->isHidden())
     	 {
     	  infobox->setText(tr("Nessuna scadenza trovata!"));
 	      infobox->exec();
 	     }
     }
   }
 }

void ARGrigliaSca::delScadenze()
 {
 	QString scadenzaID;

  if(table->currentRow()<0)
   {
    warnbox->setText(tr("Nessuna scadenza selezionata."));
 	  warnbox->exec();
 	  return;
   }
  yesnobox->setText(tr("Vuoi veramente cancellare la scadenza?"));
  if(yesnobox->exec()==QMessageBox::No) return;
  scadenzaID=table->item(table->currentRow(),0)->text();
  sql->doQuery("DELETE FROM scadenze WHERE scadenzaID='"+scadenzaID+"'");
  infobox->setText(tr("Scadenza cancellata!"));
  infobox->exec();
  visScadenze();
 }

void ARGrigliaSca::pagaScadenze()
 {
 	QString scadenzaID;

  if(table->currentRow()<0)
   {
    warnbox->setText(tr("Nessuna scadenza selezionata."));
 	  warnbox->exec();
 	  return;
   }
  yesnobox->setText(tr("Vuoi veramente segnare la scadenza come pagata?"));
  if(yesnobox->exec()==QMessageBox::No) return;
  scadenzaID=table->item(table->currentRow(),0)->text();
  sql->doQuery("UPDATE scadenze SET stato='1' WHERE scadenzaID='"+scadenzaID+"'");
  infobox->setText(tr("Scadenza segnata come pagata!"));
  infobox->exec();
  visScadenze();
 }

void ARGrigliaSca::nonPagaScadenze()
 {
 	QString scadenzaID;

  if(table->currentRow()<0)
   {
    warnbox->setText(tr("Nessuna scadenza selezionata."));
 	  warnbox->exec();
 	  return;
   }
  yesnobox->setText(tr("Vuoi veramente segnare la scadenza come non pagata?"));
  if(yesnobox->exec()==QMessageBox::No) return;
  scadenzaID=table->item(table->currentRow(),0)->text();
  sql->doQuery("UPDATE scadenze SET stato='0' WHERE scadenzaID='"+scadenzaID+"'");
  infobox->setText(tr("Scadenza segnata come non pagata!"));
  infobox->exec();
  visScadenze();
 }

void ARGrigliaSca::resRicerca()
 {
 	QDate cd = QDate::currentDate();
 	
 	populateFor("0");
 	listaStato->setCurrentIndex(0);
 	descr->clear();
 	note->clear();
 	somma->clear();
 	di->setDate(cd);
 	ckdata->setCheckState(Qt::Unchecked);
 	visScadenze();
 }

bool ARGrigliaSca::populateFor(QString val)
 {
 	MYSQL_RES *rs;
 	MYSQL_ROW row;
 	int r=0,v=0;
 	QString aux;
 	
 	listaFor->clear();
 	listaFor->addItem(tr("TUTTI I FORNITORI E LE SPESE"),QVariant(0));
 	rs=sql->doQueryRS("SELECT fornitoreID,nome FROM fornitori ORDER BY nome");
	if(rs)
 	 {
 	  while((row=mysql_fetch_row(rs)))
     {
     	listaFor->addItem(row[1],QVariant(row[0]));
   	  r++;
     	aux=row[0]; if(aux==val) v=r;
     }
    mysql_free_result(rs);
   }
  listaFor->setCurrentIndex(v);
  return r;
 }

void ARGrigliaSca::contextMenuEvent(QContextMenuEvent *event)
 {
  QMenu menu(this);
  menu.addAction(pagaCMnu);
  menu.addAction(npagCMnu);
  menu.addSeparator();
  menu.addAction(editCMnu);
  menu.addAction(delCMnu);
  menu.exec(event->globalPos());
 }

QTableWidgetItem *ARGrigliaSca::inserisciRiga(QString str)
 {
 	QTableWidgetItem *item;
 	
 	item = new QTableWidgetItem();
 	item->setText(str);
 	return item;
 }

QTableWidgetItem *ARGrigliaSca::inserisciRiga(const char *str)
 {
 	QTableWidgetItem *item;
 	
 	item = new QTableWidgetItem();
 	item->setText(str);
 	return item;
 }

QString ARGrigliaSca::quote(QString str, bool like)
 {
 	if(like)
 	 {
 	  str.replace('\\',"\\\\\\\\");
 	  str.replace('\'',"\\'");
 	 }
 	else
   {
 	  str.replace('\\',"\\\\");
 	  str.replace('\'',"\\'");
   }
 	return str;
 }
