#include <QTableWidget>
#include <QTableWidgetItem>
#include <QGroupBox>
#include <QCheckBox>
#include <QPushButton>
#include <QLineEdit>
#include <QComboBox>
#include <QLabel>
#include <QMessageBox>
#include <QDateEdit>
#include <QDate>
#include <QDialog>
#include <QInputDialog>
#include <QMenu>
#include <QAction>
#include <QContextMenuEvent>

#include "ARsql.h"
#include "ARFinestra.h"
#include "ARDialogScarica.h"
#include "ARDialogEditAll.h"
#include "ARBarraInfo.h"

#include "ARScarico.h"

ARScarico::ARScarico(QWidget *parent) : QWidget(parent)
 {
 	mainWin=(ARFinestra *)parent;
 	num_col=15;
 	rows_det=11;  // Prima colonna dettagli
 	rows_cam=7;   // Prima colonna campi
 	row_pr_pub=5; // Colonna con prezzo al pubblico
 	dettagli=false;
 	sql  = new ARsql(this);
 	sql2 = new ARsql(this);
 	sql3 = new ARsql(this);
 	scaricobox = new ARDialogScarica(parent);
 	editallbox = new ARDialogEditAll(parent);
  infobox = new QMessageBox(tr("Informazioni"),"",QMessageBox::Question,0,0,0,this);
  warnbox = new QMessageBox(tr("Avviso"),"",QMessageBox::Warning,0,0,0,this);
  yesnobox = new QMessageBox(tr("Scegli"),"",QMessageBox::Question,QMessageBox::Yes,QMessageBox::No,QMessageBox::NoButton,this);
 	cornice = new QGroupBox(tr("Ricerca articoli in magazzino "),this);

  QPalette pal(this->palette()); pal.setColor(QPalette::Base,QColor(0xff,0xff,0x99));
  _bcode  = new QLabel(tr("<b>Codice a barre</b>"),cornice); _bcode->setGeometry (  0, 15,100, 24);
  bcode   = new QLineEdit("",cornice);                       bcode->setGeometry  ( 98, 15,130, 24);
  bcode->setMaxLength(10); bcode->setPalette(pal);
  _art    = new QLabel(tr("Art."),cornice);                  _art->setGeometry   (203, 45, 80, 24);
  art     = new QLineEdit("",cornice);                       art->setGeometry    (223, 45, 45, 24);
  art->setMaxLength(255);
  _descr  = new QLabel(tr("Descr."),cornice);                _descr->setGeometry (273, 45, 80, 24);
  descr   = new QLineEdit("",cornice);                       descr->setGeometry  (308, 45,110, 24);
  descr->setMaxLength(255);
  _misura = new QLabel(tr("Misura"),cornice);                _misura->setGeometry(423, 45, 80, 24);
  misura  = new QLineEdit("",cornice);                       misura->setGeometry (463, 45, 35, 24);
  misura->setMaxLength(20);
  _prezzo = new QLabel(tr("Prezzo"),cornice);                _prezzo->setGeometry(505, 45, 80, 24);
  prezzo  = new QLineEdit("",cornice);                       prezzo->setGeometry (545, 45, 60, 24);
  prezzo->setValidator(new QDoubleValidator(-50000000.0,50000000.0,2,prezzo));
  _for = new QLabel(tr("Fornitore"),cornice);                  _for->setGeometry  (  5, 45, 80, 24);
  listaFor = new QComboBox(cornice);                         listaFor->setGeometry( 50, 45,150, 24);
  _mostra = new QLabel(tr("Visualizza"),cornice);            _mostra->setGeometry (  5, 75, 80, 24);
  listaMostra = new QComboBox(cornice);                      listaMostra->setGeometry(60,75,160,24);
  listaMostra->addItem(tr("Articoli caricati (giacenze)"),QVariant(ARSTATO_GIACENZA));
  listaMostra->addItem(tr("Articoli scaricati (venduti)"),QVariant(ARSTATO_VENDUTO));
  listaMostra->addItem(tr("Articoli resi (al fornitore)"),QVariant(ARSTATO_RESO));
  QDate cd = QDate::currentDate();
  _from = new QLabel(tr("dal"),cornice); _from->setGeometry(228, 75, 80, 24);
 	from  = new QDateEdit(cd,cornice);     from->setGeometry (253, 75, 90, 24);
 	_to   = new QLabel(tr("al"),cornice);  _to->setGeometry  (353, 75, 80, 24);
 	to    = new QDateEdit(cd,cornice);     to->setGeometry   (373, 75, 90, 24);
 	from->setDisplayFormat("dd-MMM-yyyy");
 	to->setDisplayFormat("dd-MMM-yyyy");
 	ckdata  = new QCheckBox("Disattiva ricerca per data",cornice);
 	ckdata->setGeometry(238,15,150,24);
 	ckcampi = new QCheckBox("Mostra campi",cornice);
 	ckcampi->setGeometry(613,45,150,24);
 	ckcampi->setCheckState(Qt::Checked);
 	_totali = new QLabel(tr("Totale guadagno: "),cornice); _totali->setGeometry(398,15,260,24);
 	_totali->setHidden(true);
 	
 	scarica = new QPushButton(QIcon("pic/scarico.png"),tr("Scarica"),this);
 	scarica->setIconSize(QSize(32,32));
 	rendi = new QPushButton(QIcon("pic/reso.png"),tr("Rendi merce"),this);
 	rendi->setIconSize(QSize(23,23));
 	edit = new QPushButton(QIcon("pic/edit.png"),tr("Visualizza"),this);
 	del = new QPushButton(QIcon("pic/delete.png"),tr("Cancella"),this);
 	oggi = new QPushButton(QIcon("pic/calendar.png"),"",cornice);
  oggi->setGeometry(468,75,24,24);
 	reset = new QPushButton(QIcon("pic/no_search.png"),tr("Azzera"),cornice);
  reset->setGeometry(503,83,96,24);
 	search = new QPushButton(QIcon("pic/search.png"),tr("Cerca"),cornice);
  search->setGeometry(601,83,96,24);
 	table = new QTableWidget(this);
  table->setSortingEnabled(true);
  table->setAlternatingRowColors(true);
  table->setEditTriggers(QAbstractItemView::NoEditTriggers);
  
  // TODO usare selectedItems ( una QList) per fare la modifica multipla degli articoli
  //table->setSelectionMode(QAbstractItemView::SingleSelection); // commentare questa riga se non serve pi
  table->setSelectionMode(QAbstractItemView::ExtendedSelection);
  
/*     	if(table->selectedItems().size()==2)
   	warnbox->setText(tr("iiiiiiirticolo selezionato."));
   	//QList<QTableWidgetItem *>*/
  
  table->setSelectionBehavior(QAbstractItemView::SelectRows);
 	for(int i=0;i<num_col;i++) table->insertColumn(i);
 	table->setColumnWidth(0,92);
 	table->setColumnWidth(1,60);
 	table->setColumnWidth(2,225);
 	table->setColumnWidth(3,200); // Fornitore
 	table->setColumnWidth(4,50);
 	table->setColumnWidth(5,80);
 	table->setColumnWidth(6,48);  // Sconto 
 	table->setColumnWidth(7,75);  // Colore
 	table->setColumnWidth(8,70);  // Fondo
 	table->setColumnWidth(9,45);  // Tacco
 	table->setColumnWidth(10,80); // Stagione
 	table->setColumnWidth(11,80);
 	table->setColumnWidth(12,30);
 	table->setColumnWidth(13,110);
 	table->setColumnWidth(14,80);
 	table->setHorizontalHeaderLabels(QStringList() << "Codice a barre" << "Articolo" << "Descrizione" << "Fornitore" << "Misura" << "Prezzo" << "Sconto" << "Colore" << "Fondo" << "Tacco" << "Stagione" << "Prezzo acq." << "IVA" << "Prezzo acq. ivato" << "Guadagno");
  infobar = new ARBarraInfo(this); infobar->setGeometry(0,0,100,24);
 	hidePrezzo(false);

 	edallCMnu  = new QAction(QIcon("pic/edit_all.png"),tr("Modifica tutti gli articoli selezionati"),this);
 	eticCMnu   = new QAction(QIcon("pic/etichetta.png"),tr("Stampa etichette articoli selezionati"),this);
 	stmagCMnu  = new QAction(QIcon("pic/tabulati.png"),tr("Stampa magazzino"),this);
 	editCMnu   = new QAction(QIcon("pic/edit.png"),tr("Visualizza articolo selezionato"),this);
 	delCMnu    = new QAction(QIcon("pic/delete.png"),tr("Cancella articolo selezionato"),this);
 	selallCMnu = new QAction(QIcon("pic/select_all.png"),tr("Seleziona tutto"),this);
// fare: deselezioina tutto, inverti selezione

 	connect(scarica,SIGNAL(clicked()),this,SLOT(scaricaMerce()));
 	connect(rendi,SIGNAL(clicked()),this,SLOT(resoMerce()));
 	connect(del,SIGNAL(clicked()),this,SLOT(delMagazzino()));
 	connect(delCMnu,SIGNAL(triggered()),this,SLOT(delMagazzino()));
 	connect(selallCMnu,SIGNAL(triggered()),this,SLOT(selAllMagazzino()));
 	connect(oggi,SIGNAL(clicked()),this,SLOT(settaOggi()));
 	connect(listaMostra,SIGNAL(activated(int)),this,SLOT(visMagazzino(int)));
 	connect(listaFor,SIGNAL(activated(int)),this,SLOT(visMagazzino(int)));
 	connect(search,SIGNAL(clicked()),this,SLOT(visMagazzino()));
 	connect(reset,SIGNAL(clicked()),this,SLOT(resRicerca()));
  connect(bcode,SIGNAL(returnPressed()),this,SLOT(visMagazzino()));
  connect(art,SIGNAL(returnPressed()),this,SLOT(visMagazzino()));
  connect(descr,SIGNAL(returnPressed()),this,SLOT(visMagazzino()));
  connect(misura,SIGNAL(returnPressed()),this,SLOT(visMagazzino()));
  connect(prezzo,SIGNAL(returnPressed()),this,SLOT(visMagazzino()));
  connect(ckdata,SIGNAL(stateChanged(int)),this,SLOT(visMagazzino(int)));
  connect(ckcampi,SIGNAL(stateChanged(int)),this,SLOT(hideCampi(int)));
  connect(edallCMnu,SIGNAL(triggered()),this,SLOT(editAllMagazzino()));
  // Connect per "Edit all"
  connect(editallbox->aggiorna,SIGNAL(clicked()),this,SLOT(editAllAggTutti()));
  connect(editallbox->agg_art,SIGNAL(clicked()),this,SLOT(editAllAggArt()));
  connect(editallbox->art,SIGNAL(returnPressed()),this,SLOT(editAllAggArt()));
  connect(editallbox->agg_des,SIGNAL(clicked()),this,SLOT(editAllAggDescr()));
  connect(editallbox->descr,SIGNAL(returnPressed()),this,SLOT(editAllAggDescr()));
  connect(editallbox->agg_prp,SIGNAL(clicked()),this,SLOT(editAllAggPrezzoPub()));
  connect(editallbox->prezzo_pub,SIGNAL(returnPressed()),this,SLOT(editAllAggPrezzoPub()));
  connect(editallbox->agg_pre,SIGNAL(clicked()),this,SLOT(editAllAggPrezzo()));
  connect(editallbox->prezzo,SIGNAL(returnPressed()),this,SLOT(editAllAggPrezzo()));
  connect(editallbox->agg_iva,SIGNAL(clicked()),this,SLOT(editAllAggIva()));
  connect(editallbox->iva,SIGNAL(returnPressed()),this,SLOT(editAllAggIva()));
  connect(editallbox->agg_sco,SIGNAL(clicked()),this,SLOT(editAllAggSconto()));
  connect(editallbox->sconto,SIGNAL(returnPressed()),this,SLOT(editAllAggSconto()));
  connect(editallbox->agg_for,SIGNAL(clicked()),this,SLOT(editAllAggFornitore()));

 	infobox->setWindowModality(Qt::WindowModal);
 	warnbox->setWindowModality(Qt::WindowModal);
 	yesnobox->setWindowModality(Qt::WindowModal);
 	yesnobox->setButtonText(QMessageBox::Yes,tr("Si"));
 	yesnobox->setButtonText(QMessageBox::No,tr("No"));
  hide();
 }

void ARScarico::settaGeometria(int lar, int alt)
 {
  setGeometry(0,61,lar,alt-70);
 	table->setGeometry(0,0,lar,alt-244);
 	infobar->setGeometry(0,alt-244,800,24);
 	cornice->setGeometry(2,alt-222,700,110);
  scarica->setGeometry(704,alt-213,96,49);
  rendi->setGeometry  (704,alt-213,96,49);
  edit->setGeometry   (704,alt-163,96,24);
  del->setGeometry    (704,alt-138,96,24);
 }

void ARScarico::contextMenuEvent(QContextMenuEvent *event)
 {
  QMenu menu(this);
  menu.addAction(edallCMnu);
  menu.addSeparator();
  menu.addAction(eticCMnu);
  menu.addAction(stmagCMnu);
  menu.addSeparator();
  menu.addAction(editCMnu);
  menu.addAction(delCMnu);
  menu.addSeparator();
  menu.addAction(selallCMnu);
  menu.exec(event->globalPos());
 }

void ARScarico::scaricaMerce(QString artID,bool fast)
 {
 	QString ts_sr,data_sr,ora_sr;

  // Entro nella sezione critica
  sql2->connetti();
  if(!sql2->bloccaPer(sql->getDbName()+".venditacassa","0")) // venditacassa_n
   {
  	warnbox->setText(tr("La cassa  occupata da un'altro utente.\nNon puoi scaricare l'articolo. Riprova pi tardi."));
 	  warnbox->exec();
 	  sql2->disconnetti();
 	  return;
   }
  //if(table->currentRow()<0 && artID=="")
  if(numRowSel()!=1 && artID=="") // Non posso scaricare pi di un articolo alla volta
   {
    if(numRowSel()>1) warnbox->setText(tr("Devi selezionare solo un articolo."));
    else              warnbox->setText(tr("Nessun articolo selezionato."));
 	  warnbox->exec();
   }
  else
   {
   	QString articoloID,clienteID,stato,prezzo_pub,prezzo_lis,sconto;
 	 	MYSQL_RES *rs;
    MYSQL_ROW row=0;

   	if(artID!="") articoloID=artID;
    else articoloID=table->item(currRowSel(),0)->text();
    
    // Ricava i dati dell'articolo da scaricare
    rs=sql->doQueryRS("SELECT * FROM articoli WHERE articoloID='"+articoloID+"'");
    if(rs)
     {
     	row=mysql_fetch_row(rs);
   	  if(mysql_num_rows(rs)==0)
       {
   	    warnbox->setText(tr("Articolo non trovato."));
 	      warnbox->exec();
 	      mysql_free_result(rs);
 	      sql2->disconnetti();
       	return;
       }
      mysql_free_result(rs);
     }
    stato=row[23];
    if(stato==ARSTATO_VENDUTO)
     {
     	warnbox->setText(tr("L'articolo risulta gi venduto.\nNon  possibile scaricarlo."));
 	    warnbox->exec();
 	    sql2->disconnetti();
      return;
     }
    if(stato==ARSTATO_RESO)
     {
     	warnbox->setText(tr("L'articolo risulta reso al fornitore.\nNon  possibile scaricarlo."));
 	    warnbox->exec();
 	    sql2->disconnetti();
      return;
     }
    prezzo_lis=row[18];
    sconto=row[21];
    scaricobox->settaPrezzo(prezzo_lis,sconto);
    scaricobox->settaInfo(QString(row[6]),QString(row[8]),QString(row[22]),articoloID,QString(row[1]),QString(row[32]));
    /*
    scaricobox->settaInfo(QString(row[6]),QString(row[8]),QString(row[22]),articoloID,);
    */
    
    //scaricobox->settaPrezzo(table->item(currRowSel(),5)->text());
    scaricobox->populateCli();
    
   	if(scaricobox->exec()==QDialog::Accepted)
   	 {
      prezzo_pub=scaricobox->prendiPrezzo();
      sconto=scaricobox->prendiSconto();
      clienteID=scaricobox->getClienteID();
      if(prezzo_pub.toDouble()>50000000)
       {
        warnbox->setText(tr("Prezzo troppo alto, massimo 50.000.000,00.\nArticolo non scaricato!"));
 	      warnbox->exec();
       }
      else
       {
    	  // Entro nella sezione critica
        mainWin->sql->connetti();
        if(!mainWin->sql->isBloccatoPer(sql->getDbName()+".bolle"))
         {
  	      warnbox->setText(tr("Il magazzino  occupato da un'altro utente.\nNon posso scaricare l'articolo. Riprova pi tardi."));
 	        warnbox->exec();
 	        mainWin->sql->disconnetti();
 	        sql2->disconnetti();
 	        return;
         }
        mainWin->sql->disconnetti();
        QString giacenza=ARSTATO_GIACENZA;
        stato=ARSTATO_VENDUTO;
        ts_sr=sql->timeStamp(); data_sr=sql->data(ts_sr); ora_sr=sql->ora(ts_sr);
        
        // Entro nella sezione critica
        sql3->connetti();
       	sql3->doQueryPer("LOCK TABLES articoli WRITE");
        if(sql3->doQueryPer("UPDATE articoli SET clienteID='"+clienteID+"', bolla='0', bollaID='0', fatturato='0', fatturaID='0', stato='"+stato+"', prezzo_pub='"+prezzo_pub+"', prezzo_lis='"+prezzo_lis+"', sconto='"+sconto+"', data_sr='"+data_sr+"', ora_sr='"+ora_sr+"', ts_sr='"+ts_sr+"', data_fat='0000-00-00', ora_fat='00:00:00', ts_fat='0' WHERE articoloID='"+articoloID+"' AND fatturaID='0' AND fatturato='0' AND bolla='0' AND bollaID='0' AND stato='"+giacenza+"'"))
         {
         	if(sql3->getAffected()==0)
         	 {
         	  warnbox->setText(tr("L'articolo non  stato scaricato poich\n in uso da un'altro utente!"));
         	  warnbox->exec();
         	 }
         	else
         	 {
         	  infobox->setText(tr("Articolo scaricato!"));
 	          infobox->exec();
 	         }
          sql3->doQueryPer("UNLOCK TABLES");
          sql3->disconnetti();
 	        if(!fast) visMagazzino();
         }
        else
         {
          sql3->doQueryPer("UNLOCK TABLES");
          sql3->disconnetti();
         }
       }
     }    
 	 }
 	sql2->disconnetti();
 }

void ARScarico::resoMerce()
 {
 	QString ts_sr,data_sr,ora_sr;

  if(numRowSel()!=1) // Non posso rendere pi di un articolo alla volta
   {
    if(numRowSel()>1) warnbox->setText(tr("Devi selezionare solo un articolo."));
    else              warnbox->setText(tr("Nessun articolo selezionato."));
 	  warnbox->exec();
   }
  else
   {
   	QString articoloID,stato;
   	
   	stato=listaMostra->itemData(listaMostra->currentIndex()).toString();
   	articoloID=table->item(currRowSel(),0)->text();
	 	if(stato==ARSTATO_VENDUTO) // Reso dal cliente
	 	 {
      yesnobox->setText(tr("Vuoi riprendere l'articolo dal cliente e rimetterlo in magazzino?\n(Nel caso l'articolo fosse presente in una fattura, verr rimosso anche da essa)"));
   	  if(yesnobox->exec()==QMessageBox::Yes)
   	   {
    	  // Entro nella sezione critica
        mainWin->sql->connetti();
        if(!mainWin->sql->isBloccatoPer(sql->getDbName()+".fatture"))
         {
  	      warnbox->setText(tr("Il magazzino  occupato da un'altro utente.\nNon posso rimettere l'articolo in magazzino. Riprova pi tardi."));
 	        warnbox->exec();
 	        mainWin->sql->disconnetti();
 	        return;
         }
        mainWin->sql->disconnetti();
        QString venduto=ARSTATO_VENDUTO;
        stato=ARSTATO_GIACENZA;       
        if(sql->doQuery("UPDATE articoli SET stato='"+stato+"', prezzo_pub=prezzo_lis, clienteID='0', bolla='0', bollaID='0', fatturato='0', fatturaID='0', data_sr='0000-00-00', ora_sr='00:00:00', ts_sr='0', data_fat='0000-00-00', ora_fat='00:00:00', ts_fat='0' WHERE articoloID='"+articoloID+"' AND bolla='0' AND bollaID='0' AND stato='"+venduto+"'"))
         {
         	if(sql->getAffected()==0) infobox->setText(tr("L'articolo non  stato rimesso in magazzino poich\n in uso da un'altro utente!"));
         	else infobox->setText(tr("Articolo rimesso in magazzino!"));
          infobox->exec();
          visMagazzino();
         }
       }
      return;
 	   }
	 	if(stato==ARSTATO_RESO) // Rimesso in magazzino da d.d.t.
	 	 {
      yesnobox->setText(tr("Vuoi ricaricare l'articolo in magazzino?\n (L'articolo verr rimosso anche\ndal relativo documento di trasporto)"));
   	  if(yesnobox->exec()==QMessageBox::Yes)
   	   {
    	  // Entro nella sezione critica
        mainWin->sql->connetti();
        if(!mainWin->sql->isBloccatoPer(sql->getDbName()+".bolle"))
         {
  	      warnbox->setText(tr("Il magazzino  occupato da un'altro utente.\nNon posso ricaricare l'articolo in magazzino. Riprova pi tardi."));
 	        warnbox->exec();
 	        mainWin->sql->disconnetti();
 	        return;
         }
        mainWin->sql->disconnetti();
        QString reso=ARSTATO_RESO;
        stato=ARSTATO_GIACENZA;
        if(sql->doQuery("UPDATE articoli SET stato='"+stato+"', bolla='0', bollaID='0', fatturato='0', fatturaID='0', data_sr='0000-00-00', ora_sr='00:00:00', ts_sr='0', data_fat='0000-00-00', ora_fat='00:00:00', ts_fat='0' WHERE articoloID='"+articoloID+"' AND bolla='1' AND bollaID!='0' AND fatturato='0' AND fatturaID='0' AND stato='"+reso+"'"))
         {
         	if(sql->getAffected()==0) infobox->setText(tr("L'articolo non  stato ricaricato in magazzino poich\n in uso da un'altro utente!"));
         	else infobox->setText(tr("Articolo ricaricato in magazzino!"));
          infobox->exec();
          visMagazzino();
         }
       }
      return;
 	   }
   }
 }

void ARScarico::delMagazzino()
 {
 	MYSQL_RES *rs;
 	QString articoloID;

  if(numRowSel()!=1) // Non posso cancellare pi di un articolo alla volta
   {
    if(numRowSel()>1) warnbox->setText(tr("Devi selezionare solo un articolo."));
    else              warnbox->setText(tr("Nessun articolo selezionato."));
 	  warnbox->exec();
    return;
   }
  yesnobox->setText(tr("Vuoi veramente cancellare l'articolo selezionato?"));
  if(yesnobox->exec()==QMessageBox::No) return;
  articoloID=table->item(currRowSel(),0)->text();
  rs=sql->doQueryRS("SELECT bollaID FROM articoli WHERE bollaID!='0' AND articoloID='"+articoloID+"'");
  if(rs)
   {
   	if(mysql_num_rows(rs)!=0)
     {
   	  warnbox->setText(tr("Non posso cancellare l'articolo poich\n presente in un documento di trasporto."));
      warnbox->exec();
      mysql_free_result(rs);
      return;
     }
    mysql_free_result(rs);
   }
  QString stato=ARSTATO_VENDUTO;
  rs=sql->doQueryRS("SELECT bollaID FROM articoli WHERE stato='"+stato+"' AND articoloID='"+articoloID+"'");
  if(rs)
   {
   	if(mysql_num_rows(rs)!=0)
     {
   	  warnbox->setText(tr("Non posso cancellare l'articolo poich\nl'articolo risulta venduto."));
      warnbox->exec();
      mysql_free_result(rs);
      return;
     }
    mysql_free_result(rs);
   }
  // Entro nella sezione critica
  mainWin->sql->connetti();
  if(!mainWin->sql->isBloccatoPer(sql->getDbName()+".bolle"))
   {
    warnbox->setText(tr("Il magazzino  occupato da un'altro utente.\nNon posso cancellare l'articolo. Riprova pi tardi."));
 	  warnbox->exec();
 	  mainWin->sql->disconnetti();
 	  return;
   }
  mainWin->sql->disconnetti();
  sql->doQuery("DELETE FROM articoli WHERE articoloID='"+articoloID+"'");
  infobox->setText(tr("Articolo cancellato!"));
  infobox->exec();
  visMagazzino();
 }

void ARScarico::selAllMagazzino() { table->selectAll(); }

void ARScarico::resRicerca()
 {
 	QDate cd = QDate::currentDate();
 	
 	from->setDate(cd);
 	to->setDate(cd);
 	bcode->clear();
 	art->clear();
 	descr->clear();
 	misura->clear();
 	prezzo->clear();
 	ckdata->setCheckState(Qt::Unchecked);
 	ckcampi->setCheckState(Qt::Checked);
 	populateFor("0");
 }

void ARScarico::settaOggi()
 {
 	QDate cd = QDate::currentDate();
 	
 	from->setDate(cd);
 	to->setDate(cd);
 	ckdata->setCheckState(Qt::Unchecked);
 }

void ARScarico::editAllMagazzino()
 {
 	QString art,descr,prezzo_pub,prezzo,iva,sconto,forID;
  MYSQL_RES *rs;
  MYSQL_ROW row;
 	
 	if(numRowSel()==0) // Nessuna riga selezionata
   {
    warnbox->setText(tr("Devi selezionare almeno un articolo."));
 	  warnbox->exec();
   }
  else
   {
  	art=table->item(currRowSel(),1)->text();
 	  descr=table->item(currRowSel(),2)->text();
 	  prezzo_pub=table->item(currRowSel(),row_pr_pub)->text();
 	  prezzo=table->item(currRowSel(),rows_det)->text();
 	  iva=table->item(currRowSel(),rows_det+1)->text();
 	  sconto=table->item(currRowSel(),row_pr_pub+1)->text();
    rs=sql->doQueryRS("SELECT fornitoreID FROM articoli WHERE articoloID='"+table->item(currRowSel(),0)->text()+"'");
	  if(rs)
 	   {
 	   	row=mysql_fetch_row(rs);
 	    forID=row[0];
      mysql_free_result(rs);
     }
    else forID="0";
 	  editallbox->settaValori(art,descr,prezzo_pub,prezzo,iva,sconto,forID,dettagli);
 	  editallbox->exec();
   }
 }

void ARScarico::editAllAggTutti()
 {
 	QString articoloID,art,descr,prezzo_pub,prezzo,iva,sconto,fornitoreID;
 	bool ok;
 	double d;

  // Entro nella sezione critica (forse qui e' superfluo, controllare)
  sql2->connetti();
  if(!sql2->bloccaPer(sql->getDbName()+".venditacassa","0")) // .venditacassa_n
   {
    editallbox->warnbox->setText(tr("Un'altro utente sta usando gli articoli,\nnon posso modificare i campi degli articoli. Riprova pi tardi."));
 	  editallbox->warnbox->exec();
    sql2->disconnetti();
    return;
   }
  sql2->disconnetti();
 	if(table->rowCount()==0)
   {
   	editallbox->infobox->setText(tr("Nessun articolo presente!"));
   	editallbox->infobox->exec();
   	return;
   }
  if(editallbox->getArt()=="")
   {
   	editallbox->infobox->setText(tr("Codice articolo vuoto, inserire codice articolo!\nArticoli non aggiornati!"));
   	editallbox->infobox->exec();
   	return;
   }
  if(editallbox->getDescr()=="")
   {
   	editallbox->infobox->setText(tr("Descrizione vuota, inserire descrizione!\nArticoli non aggiornati!"));
   	editallbox->infobox->exec();
   	return;
   }
  d=editallbox->getPrezzoPub().toDouble(&ok);
 	if(!ok || d<0 || d>50000000)
   {
   	editallbox->infobox->setText(tr("Formato prezzo al pubblico errato,\nvalore negativo o\nvalore superiore a 50.000.000,00\nArticoli non aggiornati!"));
   	editallbox->infobox->exec();
   	return;
   }
  if(editallbox->getSconto()=="")
   {
   	editallbox->infobox->setText(tr("Campo sconto vuoto, inserire la percentuale di sconto!\nArticoli non aggiornati!"));
   	editallbox->infobox->exec();
   	return;
   }
 	d=editallbox->getPrezzo().toDouble(&ok);
 	if(dettagli)
 	 {
 	  if(!ok || d<0 || d>50000000)
     {
   	  editallbox->infobox->setText(tr("Formato prezzo d'acquisto errato,\nvalore negativo o\nvalore superiore a 50.000.000,00\nArticoli non aggiornati!"));
   	  editallbox->infobox->exec();
   	  return;
     }
    if(editallbox->getIva()=="")
     {
   	  editallbox->infobox->setText(tr("Campo IVA vuoto, inserire IVA!\nArticoli non aggiornati!"));
   	  editallbox->infobox->exec();
   	  return;
     }
   }
   
  editallbox->yesnobox->setText(tr("Vuoi veramente aggiornare tutti i campi degli articoli?"));
  if(editallbox->yesnobox->exec()==QMessageBox::No) return;

 	fornitoreID = editallbox->getFornitoreID();
  // Entro nella sezione critica
 	mainWin->sql->connetti();
 	mainWin->sql->doQueryPer("LOCK TABLES articoli WRITE");
  if(!editallbox->esisteFornitore(fornitoreID))
   {
   	editallbox->infobox->setText(tr("Fornitore inesistente, verificare fornitore!\nArticoli non aggiornati!"));
   	editallbox->infobox->exec();
    mainWin->sql->doQueryPer("UNLOCK TABLES");
    mainWin->sql->disconnetti();
    return;
   }
 	art = editallbox->getArt();
 	descr = editallbox->getDescr();
 	prezzo_pub = editallbox->getPrezzoPub();
 	prezzo = editallbox->getPrezzo();
 	iva = editallbox->getIva();
 	sconto = editallbox->getSconto();
 	editallbox->mostraProgresso(true);
 	int dd=0;
 	for(int r=0;r<table->rowCount();r++)
 	 {
    if(table->isItemSelected(table->item(r,0)))
     {
      QString giacenza=ARSTATO_GIACENZA;
 	 	  articoloID = table->item(r,0)->text();
      if(dettagli) mainWin->sql->doQueryPer("UPDATE articoli SET codice='"+art+"', descrizione='"+descr+"', prezzo_pub='"+prezzo_pub+"', prezzo_lis='"+prezzo_pub+"', sconto='"+sconto+"', prezzo='"+prezzo+"', iva='"+iva+"', fornitoreID='"+fornitoreID+"' WHERE articoloID='"+articoloID+"' AND stato='"+giacenza+"' AND fatturaID='0' AND bollaID='0'");
      else mainWin->sql->doQueryPer("UPDATE articoli SET codice='"+art+"', descrizione='"+descr+"', prezzo_pub='"+prezzo_pub+"', prezzo_lis='"+prezzo_pub+"', sconto='"+sconto+"', fornitoreID='"+fornitoreID+"' WHERE articoloID='"+articoloID+"' AND stato='"+giacenza+"' AND fatturaID='0' AND bollaID='0'");
      editallbox->settaProgresso((int)(((float)(dd+1)/(float)numRowSel())*100));
      dd++;
     }
   }
  mainWin->sql->doQueryPer("UNLOCK TABLES");
  mainWin->sql->disconnetti();
  
  if(quote(this->art->text())!="")   this->art->setText(art);
  if(quote(this->descr->text())!="") this->descr->setText(descr);
  if(this->prezzo->text()!="")       this->prezzo->setText(prezzo_pub);
  if(this->listaFor->currentIndex()!=0) this->listaFor->setCurrentIndex(this->listaFor->findData(QVariant(fornitoreID)));
  
 	visMagazzino();
 	editallbox->infobox->setText(tr("Articoli aggiornati!"));
 	editallbox->infobox->exec();
 	editallbox->mostraProgresso(false);
 }

void ARScarico::editAllAggArt()
 {
 	QString articoloID,art;

  if(editallbox->getArt()=="")
   {
   	editallbox->infobox->setText(tr("Codice articolo vuoto, inserire codice articolo!\nArticoli non aggiornati!"));
   	editallbox->infobox->exec();
   	return;
   }
  editallbox->yesnobox->setText(tr("Vuoi veramente aggiornare tutti i campi articolo?"));
  if(editallbox->yesnobox->exec()==QMessageBox::No) return; 	
 	art = editallbox->getArt();
 	editallbox->mostraProgresso(true);

  // Entro nella sezione critica
 	mainWin->sql->connetti();
 	mainWin->sql->doQueryPer("LOCK TABLES articoli WRITE"); 	
  int d=0;
  for(int r=0;r<table->rowCount();r++)
 	 {
 	  if(table->isItemSelected(table->item(r,0)))
     {
      QString giacenza=ARSTATO_GIACENZA;
 	 	  articoloID = table->item(r,0)->text();
      mainWin->sql->doQueryPer("UPDATE articoli SET codice='"+art+"' WHERE articoloID='"+articoloID+"' AND stato='"+giacenza+"' AND fatturaID='0' AND bollaID='0'");
      // fare messaggio: if numero affected < rowCount -> alcuni articoli non sono stati aggiornati poiche in uso...
      editallbox->settaProgresso((int)(((float)(d+1)/(float)numRowSel())*100));
      d++;
     }
   }
  mainWin->sql->doQueryPer("UNLOCK TABLES");
  mainWin->sql->disconnetti();
   
  if(quote(this->art->text())!="") this->art->setText(art);
 	visMagazzino();
 	editallbox->infobox->setText(tr("Articoli aggiornati!"));
 	editallbox->infobox->exec();
 	editallbox->mostraProgresso(false);
 }

void ARScarico::editAllAggDescr()
 {
 	QString articoloID,descr;

  if(editallbox->getDescr()=="")
   {
   	editallbox->infobox->setText(tr("Descrizione vuota, inserire descrizione!\nArticoli non aggiornati!"));
   	editallbox->infobox->exec();
   	return;
   }
  editallbox->yesnobox->setText(tr("Vuoi veramente aggiornare tutti i campi descrizione?"));
  if(editallbox->yesnobox->exec()==QMessageBox::No) return;
 	descr = editallbox->getDescr();
 	editallbox->mostraProgresso(true);

  // Entro nella sezione critica
 	mainWin->sql->connetti();
 	mainWin->sql->doQueryPer("LOCK TABLES articoli WRITE");
 	int d=0;
  for(int r=0;r<table->rowCount();r++)
 	 {
 	 	if(table->isItemSelected(table->item(r,0)))
 	 	 {
      QString giacenza=ARSTATO_GIACENZA;
 	 	  articoloID = table->item(r,0)->text();
      mainWin->sql->doQueryPer("UPDATE articoli SET descrizione='"+descr+"' WHERE articoloID='"+articoloID+"' AND stato='"+giacenza+"' AND fatturaID='0' AND bollaID='0'");
      editallbox->settaProgresso((int)(((float)(d+1)/(float)numRowSel())*100));
      d++;
     }
   }
  mainWin->sql->doQueryPer("UNLOCK TABLES");
  mainWin->sql->disconnetti();

  if(quote(this->descr->text())!="") this->descr->setText(descr); 
 	visMagazzino();
 	editallbox->infobox->setText(tr("Articoli aggiornati!"));
 	editallbox->infobox->exec();
 	editallbox->mostraProgresso(false);
 }

void ARScarico::editAllAggPrezzoPub()
 {
 	QString articoloID,prezzo_pub;
 	bool ok;
 	double d;

  // Entro nella sezione critica (forse qui e' superfluo, controllare)
  sql2->connetti();
  if(!sql2->bloccaPer(sql->getDbName()+".venditacassa","0")) // .venditacassa_n
   {
    editallbox->warnbox->setText(tr("Un'altro utente sta usando gli articoli,\nnon posso modificare il prezzo al pubblico degli articoli. Riprova pi tardi."));
 	  editallbox->warnbox->exec();
    sql2->disconnetti();
    return;
   }
  sql2->disconnetti();
 	d=editallbox->getPrezzoPub().toDouble(&ok);
 	if(!ok || d<0 || d>50000000)
   {
   	editallbox->infobox->setText(tr("Formato prezzo al pubblico errato,\nvalore negativo o\nvalore superiore a 50.000.000,00\nArticoli non aggiornati!"));
   	editallbox->infobox->exec();
   	return;
   }
  editallbox->yesnobox->setText(tr("Vuoi veramente aggiornare tutti i prezzi al pubblico?"));
  if(editallbox->yesnobox->exec()==QMessageBox::No) return; 	
 	prezzo_pub = editallbox->getPrezzoPub();
 	editallbox->mostraProgresso(true);
 	
  // Entro nella sezione critica
 	mainWin->sql->connetti();
 	mainWin->sql->doQueryPer("LOCK TABLES articoli WRITE");
 	int dd=0;
  for(int r=0;r<table->rowCount();r++)
 	 {
    if(table->isItemSelected(table->item(r,0)))
     {
 	 	  QString giacenza=ARSTATO_GIACENZA;
 	 	  articoloID = table->item(r,0)->text();
      mainWin->sql->doQueryPer("UPDATE articoli SET prezzo_pub='"+prezzo_pub+"', prezzo_lis='"+prezzo_pub+"' WHERE articoloID='"+articoloID+"' AND stato='"+giacenza+"' AND fatturaID='0' AND bollaID='0'");
      editallbox->settaProgresso((int)(((float)(dd+1)/(float)numRowSel())*100));
      dd++;
     }
   }
  mainWin->sql->doQueryPer("UNLOCK TABLES");
  mainWin->sql->disconnetti();
  
  if(this->prezzo->text()!="") this->prezzo->setText(prezzo_pub);
 	visMagazzino();
 	editallbox->infobox->setText(tr("Articoli aggiornati!"));
 	editallbox->infobox->exec();
 	editallbox->mostraProgresso(false);
 }

void ARScarico::editAllAggPrezzo()
 {
 	QString articoloID,prezzo;
 	bool ok;
 	double d;

 	d=editallbox->getPrezzo().toDouble(&ok);
 	if(!ok || d<0 || d>50000000)
   {
   	editallbox->infobox->setText(tr("Formato prezzo d'acquisto errato,\nvalore negativo o\nvalore superiore a 50.000.000,00\nArticoli non aggiornati!"));
   	editallbox->infobox->exec();
   	return;
   }
  editallbox->yesnobox->setText(tr("Vuoi veramente aggiornare tutti i prezzi d'acquisto?"));
  if(editallbox->yesnobox->exec()==QMessageBox::No) return; 	
 	prezzo = editallbox->getPrezzo();
 	editallbox->mostraProgresso(true);
 	
  // Entro nella sezione critica
 	mainWin->sql->connetti();
 	mainWin->sql->doQueryPer("LOCK TABLES articoli WRITE");
 	int dd=0;	
  for(int r=0;r<table->rowCount();r++)
 	 {
    if(table->isItemSelected(table->item(r,0)))
     {
 	 	  QString giacenza=ARSTATO_GIACENZA;
 	 	  articoloID = table->item(r,0)->text();
      mainWin->sql->doQueryPer("UPDATE articoli SET prezzo='"+prezzo+"' WHERE articoloID='"+articoloID+"' AND stato='"+giacenza+"' AND fatturaID='0' AND bollaID='0'");
      editallbox->settaProgresso((int)(((float)(dd+1)/(float)numRowSel())*100));
      dd++;
     }
   }
  mainWin->sql->doQueryPer("UNLOCK TABLES");
  mainWin->sql->disconnetti();

 	visMagazzino();
 	editallbox->infobox->setText(tr("Articoli aggiornati!"));
 	editallbox->infobox->exec();
 	editallbox->mostraProgresso(false);
 }

void ARScarico::editAllAggIva()
 {
 	QString articoloID,iva;

  if(editallbox->getIva()=="")
   {
   	editallbox->infobox->setText(tr("Campo IVA vuoto, inserire IVA!\nArticoli non aggiornati!"));
   	editallbox->infobox->exec();
   	return;
   }
  editallbox->yesnobox->setText(tr("Vuoi veramente aggiornare tutti i valori dell'IVA?"));
  if(editallbox->yesnobox->exec()==QMessageBox::No) return; 	
 	iva = editallbox->getIva();
 	editallbox->mostraProgresso(true);
 	
  // Entro nella sezione critica
 	mainWin->sql->connetti();
 	mainWin->sql->doQueryPer("LOCK TABLES articoli WRITE");
 	int d=0;
  for(int r=0;r<table->rowCount();r++)
 	 {
    if(table->isItemSelected(table->item(r,0)))
     {
 	 	  QString giacenza=ARSTATO_GIACENZA;
 	 	  articoloID = table->item(r,0)->text();
      mainWin->sql->doQueryPer("UPDATE articoli SET iva='"+iva+"' WHERE articoloID='"+articoloID+"' AND stato='"+giacenza+"' AND fatturaID='0' AND bollaID='0'");
      editallbox->settaProgresso((int)(((float)(d+1)/(float)numRowSel())*100));
      d++;
     }
   }
  mainWin->sql->doQueryPer("UNLOCK TABLES");
  mainWin->sql->disconnetti();
  
  visMagazzino(); 
 	editallbox->infobox->setText(tr("Articoli aggiornati!"));
 	editallbox->infobox->exec();
 	editallbox->mostraProgresso(false);
 }

void ARScarico::editAllAggSconto()
 {
 	QString articoloID,sconto;
 	
  // Entro nella sezione critica (forse qui e' superfluo, controllare)
  sql2->connetti();
  if(!sql2->bloccaPer(sql->getDbName()+".venditacassa","0")) // .venditacassa_n
   {
    editallbox->warnbox->setText(tr("Un'altro utente sta usando gli articoli,\nnon posso modificare il prezzo al pubblico degli articoli. Riprova pi tardi."));
 	  editallbox->warnbox->exec();
    sql2->disconnetti();
    return;
   }
  sql2->disconnetti();

  if(editallbox->getSconto()=="")
   {
   	editallbox->infobox->setText(tr("Campo sconto vuoto, inserire la percentuale di sconto!\nArticoli non aggiornati!"));
   	editallbox->infobox->exec();
   	return;
   }
  editallbox->yesnobox->setText(tr("Vuoi veramente aggiornare tutte le percentuali di sconto?"));
  if(editallbox->yesnobox->exec()==QMessageBox::No) return; 	
 	sconto = editallbox->getSconto();
 	editallbox->mostraProgresso(true);
 	
  // Entro nella sezione critica
 	mainWin->sql->connetti();
 	mainWin->sql->doQueryPer("LOCK TABLES articoli WRITE");
 	int d=0;
  for(int r=0;r<table->rowCount();r++)
 	 {
    if(table->isItemSelected(table->item(r,0)))
     { 	 	
 	 	  QString giacenza=ARSTATO_GIACENZA;
 	 	  articoloID = table->item(r,0)->text();
      mainWin->sql->doQueryPer("UPDATE articoli SET sconto='"+sconto+"' WHERE articoloID='"+articoloID+"' AND stato='"+giacenza+"' AND fatturaID='0' AND bollaID='0'");
      editallbox->settaProgresso((int)(((float)(d+1)/(float)numRowSel())*100));
      d++;
     }
   }
  mainWin->sql->doQueryPer("UNLOCK TABLES");
  mainWin->sql->disconnetti();
  
  visMagazzino();
 	editallbox->infobox->setText(tr("Articoli aggiornati!"));
 	editallbox->infobox->exec();
 	editallbox->mostraProgresso(false);
 }

void ARScarico::editAllAggFornitore()
 {
 	QString articoloID,fornitoreID;

  editallbox->yesnobox->setText(tr("Vuoi veramente aggiornare tutti i campi fornitore?"));
  if(editallbox->yesnobox->exec()==QMessageBox::No) return;
  
 	fornitoreID = editallbox->getFornitoreID();
  // Entro nella sezione critica
 	mainWin->sql->connetti();
 	mainWin->sql->doQueryPer("LOCK TABLES articoli WRITE");
  if(!editallbox->esisteFornitore(fornitoreID))
   {
   	editallbox->infobox->setText(tr("Fornitore inesistente, verificare fornitore!\nArticoli non aggiornati!"));
   	editallbox->infobox->exec();
    mainWin->sql->doQueryPer("UNLOCK TABLES");
    mainWin->sql->disconnetti();
    return;
   }
  editallbox->mostraProgresso(true);
  int d=0;
  for(int r=0;r<table->rowCount();r++)
 	 {
 	 	if(table->isItemSelected(table->item(r,0)))
 	 	 {
      QString giacenza=ARSTATO_GIACENZA;
 	 	  articoloID = table->item(r,0)->text();
      mainWin->sql->doQueryPer("UPDATE articoli SET fornitoreID='"+fornitoreID+"' WHERE articoloID='"+articoloID+"' AND stato='"+giacenza+"' AND fatturaID='0' AND bollaID='0'");
      editallbox->settaProgresso((int)(((float)(d+1)/(float)numRowSel())*100));
      d++;
     }
   }
  mainWin->sql->doQueryPer("UNLOCK TABLES");
  mainWin->sql->disconnetti();

  if(this->listaFor->currentIndex()!=0) this->listaFor->setCurrentIndex(this->listaFor->findData(QVariant(fornitoreID)));
 	visMagazzino();
 	editallbox->infobox->setText(tr("Articoli aggiornati!"));
 	editallbox->infobox->exec();
 	editallbox->mostraProgresso(false);
 }

void ARScarico::visMagazzino(int) {	visMagazzino(); } // Wrapper per evento "cambio listaMostra"

void ARScarico::visMagazzino()
 {
 	MYSQL_RES *rs;
 	MYSQL_ROW row;
 	QString query,stato,bcode,art,descr,misura,prezzo,fornitoreID;
 	bool primo;
 	int r;

  populateFor(listaFor->itemData(listaFor->currentIndex()).toString());
 	table->setRowCount(0);
 	r=0; primo=true;
 	query="SELECT articoloID,codice,descrizione,fornitori.nome,misura,prezzo_pub,sconto,colore,fondo,tacco,stagione,prezzo,iva,fatturaID FROM articoli,fornitori";
 	bcode=quote(this->bcode->text(),true);
 	art=quote(this->art->text(),true);
 	descr=quote(this->descr->text(),true);
 	misura=quote(this->misura->text());
 	prezzo=quote(this->prezzo->text(),true);
 	fornitoreID=listaFor->itemData(listaFor->currentIndex()).toString();
  if(bcode!="")  { if(primo) { query+=" WHERE articoloID='"+bcode+"'"; primo=false; } else { query+=" AND articoloID='"+bcode+"'"; } }
 	if(art!="")    { if(primo) { query+=" WHERE codice LIKE '%"+art+"%'"; primo=false; } else { query+=" AND codice LIKE '%"+art+"%'"; } }
 	if(descr!="")  { if(primo) { query+=" WHERE descrizione LIKE '%"+descr+"%'"; primo=false; } else { query+=" AND descrizione LIKE '%"+descr+"%'"; } }
 	if(misura!="") { if(primo) { query+=" WHERE misura='"+misura+"'"; primo=false; } else { query+=" AND misura='"+misura+"'"; } }
 	if(prezzo!="") { if(primo) { query+=" WHERE prezzo_pub LIKE '%"+prezzo+"%'"; primo=false; } else { query+=" AND prezzo_pub LIKE '%"+prezzo+"%'"; } }
 	if(fornitoreID!="0") { if(primo) { query+=" WHERE articoli.fornitoreID='"+fornitoreID+"'"; primo=false; } else { query+=" AND articoli.fornitoreID='"+fornitoreID+"'"; } }
 	
 	if(primo) { query+=" WHERE articoli.fornitoreID=fornitori.fornitoreID"; primo=false; } else { query+=" AND articoli.fornitoreID=fornitori.fornitoreID"; }
  
  stato=listaMostra->itemData(listaMostra->currentIndex()).toString();
  if(stato==ARSTATO_VENDUTO || stato==ARSTATO_RESO) { edallCMnu->setEnabled(false); }
  else edallCMnu->setEnabled(true);
 	if(primo) { query+=" WHERE stato='"+stato+"'"; primo=false; } else { query+=" AND stato='"+stato+"'"; }
 	if(stato==ARSTATO_GIACENZA)
 	 {
 	 	scarica->setVisible(true);
 	 	rendi->setVisible(false);
 	 	if(ckdata->checkState()==Qt::Unchecked)
 	   if(primo) { query+=" WHERE (articoli.data >= '"+from->date().toString(Qt::ISODate)+"' AND articoli.data <= '"+to->date().toString(Qt::ISODate)+"')"; primo=false; } else { query+=" AND (articoli.data >= '"+from->date().toString(Qt::ISODate)+"' AND articoli.data <= '"+to->date().toString(Qt::ISODate)+"')"; }
 	  table->hideColumn(rows_det+3); _totali->setHidden(true);
 	 }
 	if(stato==ARSTATO_VENDUTO)
 	 {
 	 	scarica->setVisible(false);
 	 	rendi->setText(tr("Reso cliente")); rendi->setVisible(true);
 	 	if(ckdata->checkState()==Qt::Unchecked)
 	 	 if(primo) { query+=" WHERE (articoli.data_sr >= '"+from->date().toString(Qt::ISODate)+"' AND data_sr <= '"+to->date().toString(Qt::ISODate)+"')"; primo=false; } else { query+=" AND (data_sr >= '"+from->date().toString(Qt::ISODate)+"' AND articoli.data_sr <= '"+to->date().toString(Qt::ISODate)+"')"; }
 	 	if(dettagli) { table->showColumn(rows_det+3); _totali->setText(calcGuadagno()); _totali->setHidden(false); }
 	 	else         { table->hideColumn(rows_det+3); _totali->setHidden(true); }
 	 }
 	if(stato==ARSTATO_RESO)
 	 {
 	 	scarica->setVisible(false);
 	 	rendi->setText(tr("Al magazzino")); rendi->setVisible(true);
 	 	if(ckdata->checkState()==Qt::Unchecked)
 	 	 if(primo) { query+=" WHERE (articoli.data_sr >= '"+from->date().toString(Qt::ISODate)+"' AND data_sr <= '"+to->date().toString(Qt::ISODate)+"')"; primo=false; } else { query+=" AND (data_sr >= '"+from->date().toString(Qt::ISODate)+"' AND articoli.data_sr <= '"+to->date().toString(Qt::ISODate)+"')"; }
 	 	table->hideColumn(rows_det+3); _totali->setHidden(true);
 	 }
 	query+=" ORDER BY articoli.ts";
 	rs=sql->doQueryRS(query);
 	if(rs)
 	 {
 	  while((row=mysql_fetch_row(rs)))
     {
   	  table->insertRow(r); table->setRowHeight(r,18);
   	  for(int c=0;c<num_col-2;c++) { if(c!=rows_det && c!=rows_det+1) table->setItem(r,c,inserisciRiga(row[c])); else table->setItem(r,c,inserisciRiga(row[c],Qt::AlignRight)); }
   	  // La merce fatturata in azzurro
   	  if(QString(row[13])!="0") table->item(r,0)->setBackgroundColor(QColor(140,255,255));
   	  QString aux_iva=row[num_col-3];
   	  QString aux_prezzo=row[num_col-4];
   	  double ivato=aux_prezzo.toDouble()+(aux_prezzo.toDouble()*aux_iva.toDouble()/100);
   	  QString aux; aux.setNum(ivato,'F',2);
   	  table->setItem(r,num_col-2,inserisciRiga(aux,Qt::AlignRight)); // Prezzo ivato
   	  QString aux_prezzo_pub=row[row_pr_pub];
   	  double guad=aux_prezzo_pub.toDouble()-ivato;
   	  aux.setNum(guad,'F',2);
   	  table->setItem(r,num_col-1,inserisciRiga(aux,Qt::AlignRight)); // Guadagno
   	  r++;
     }
    infobar->setTesto(calcTotArticoli(),1);
    infobar->setTesto(calcTotPrezzo(),2);
    infobar->setTesto(calcTotPrezzoAcq(),3);
    infobar->setTesto(calcTotIVA(),4);
    infobar->setTesto(calcTotPrezzoIVA(),5);
    this->bcode->setFocus();
    mysql_free_result(rs);
    if(r==0) 
     {
     	infobox->setText(tr("Nessun articolo trovato!"));
 	    infobox->exec();
     }
   }
  if(stato==ARSTATO_VENDUTO)
   {
 	 	if(dettagli) { table->showColumn(rows_det+3); _totali->setText(calcGuadagno()); _totali->setHidden(false); }
  	else         { table->hideColumn(rows_det+3); _totali->setHidden(true); }
   }
 }

void ARScarico::hidePrezzo(bool fl)
 {
 	if(!fl)
 	 {
 	  table->hideColumn(rows_det);
	  table->hideColumn(rows_det+1);
    table->hideColumn(rows_det+2);
    table->hideColumn(rows_det+3);
    _totali->setHidden(true);
    dettagli=false;
    infobar->hidePrezzo(true);
   }
  else
   {
   	QString stato,pwd;
   	bool ok;

    pwd=QInputDialog::getText(this,tr("Inserisci la password"),tr("Password"),QLineEdit::Password,"",&ok,Qt::Window | Qt::MSWindowsFixedSizeDialogHint | Qt::WindowTitleHint | Qt::WindowSystemMenuHint);
   	if(ok && pwd==sql->prendiPwd())
   	 {
   	  stato=listaMostra->itemData(listaMostra->currentIndex()).toString();
 	    table->showColumn(rows_det);
	    table->showColumn(rows_det+1);
      table->showColumn(rows_det+2);
      if(stato==ARSTATO_VENDUTO)
       {
     	  table->showColumn(rows_det+3);
     	  _totali->setText(calcGuadagno());
     	  _totali->setHidden(false);
       }
      dettagli=true;
      infobar->hidePrezzo(false);
     }
    else
     {
      warnbox->setText(tr("Password errata, accesso negato!"));
 	    warnbox->exec();
 	    dettagli=false;
     }
   }
 }

bool ARScarico::populateFor(QString val)
 {
 	MYSQL_RES *rs;
 	MYSQL_ROW row;
 	int r=0,v=0;
 	QString aux;
 	
 	listaFor->clear();
 	listaFor->addItem(tr("TUTTI I FORNITORI"),QVariant(0));
 	rs=sql->doQueryRS("SELECT fornitoreID,nome FROM fornitori ORDER BY nome");
	if(rs)
 	 {
 	  while((row=mysql_fetch_row(rs)))
     {
     	listaFor->addItem(row[1],QVariant(row[0]));
   	  r++;
     	aux=row[0]; if(aux==val) v=r;
     }
    mysql_free_result(rs);
   }
  listaFor->setCurrentIndex(v);
  return r;
 }

void ARScarico::hideCampi(int st)
 {
 	if(st==Qt::Unchecked)
 	 {
    table->hideColumn(rows_cam);
	  table->hideColumn(rows_cam+1);
    table->hideColumn(rows_cam+2);
    table->hideColumn(rows_cam+3);
   }
  else
   {
   	table->showColumn(rows_cam);
	  table->showColumn(rows_cam+1);
    table->showColumn(rows_cam+2);
    table->showColumn(rows_cam+3);
   }
 }

QTableWidgetItem *ARScarico::inserisciRiga(const char *str, int alig)
 {
 	QTableWidgetItem *item;
 	
 	item = new QTableWidgetItem();
 	item->setText(str);
 	item->setTextAlignment(alig);
 	return item;
 }

QTableWidgetItem *ARScarico::inserisciRiga(QString str, int alig)
 {
 	QTableWidgetItem *item;
 	
 	item = new QTableWidgetItem();
 	item->setText(str);
 	item->setTextAlignment(alig);
 	return item;
 }

int ARScarico::numRowSel(int col)
 {
 	// Prende il numero di righe selezionate nella griglia (table)
 	int num_row_sel=0;
 	
 	for(int r=0;r<table->rowCount();r++)
 	 if(table->isItemSelected(table->item(r,col))) num_row_sel++;
 	return num_row_sel;
 }

int ARScarico::currRowSel(int col)
 {
 	// Prende l'indice della riga selezionata nella griglia (table) (vale nel caso ci sia solo una riga selezionata)
 	int cur_row_sel=0;
 	
 	for(int r=0;r<table->rowCount();r++)
 	 if(table->isItemSelected(table->item(r,col))) cur_row_sel=r;
 	return cur_row_sel;
 }

QString ARScarico::calcGuadagno()
 {
 	QString sguadagno;
 	double guadagno=0;

 	for(int r=0;r<table->rowCount();r++)
 	 {
 	 	if(table->item(r,rows_det+3)) guadagno=guadagno+table->item(r,rows_det+3)->text().toDouble();
 	 }
 	sguadagno.sprintf("Totale guadagno: %.2lf Euro (%d pezzi venduti)",guadagno,table->rowCount());
 	return sguadagno;
 }

QString ARScarico::calcTotArticoli()
 {
 	QString aux;
 	int val;

 	val=table->rowCount();
 	aux.sprintf("%d",val);
 	return aux;
 }

QString ARScarico::calcTotPrezzo()
 {
 	QString aux;
 	double val=0;

 	for(int r=0;r<table->rowCount();r++)
 	 {
 	 	if(table->item(r,row_pr_pub)) val=val+table->item(r,row_pr_pub)->text().toDouble();
 	 }
 	aux.sprintf("%.2lf",val);
 	return aux;
 }

QString ARScarico::calcTotPrezzoAcq()
 {
 	QString aux;
 	double val=0;

 	for(int r=0;r<table->rowCount();r++)
 	 {
 	 	if(table->item(r,rows_det)) val=val+table->item(r,rows_det)->text().toDouble();
 	 }
 	aux.sprintf("%.2lf",val);
 	return aux;
 }

QString ARScarico::calcTotIVA()
 {
 	QString aux;
 	double val1=0,val2=0,val=0;

 	for(int r=0;r<table->rowCount();r++)
 	 {
 	 	if(table->item(r,rows_det+2)) val1=val1+table->item(r,rows_det+2)->text().toDouble();
 	 	if(table->item(r,rows_det))   val2=val2+table->item(r,rows_det)->text().toDouble();
 	 }
 	val=val1-val2;
 	aux.sprintf("%.2lf",val);
 	return aux;
 }

QString ARScarico::calcTotPrezzoIVA()
 {
 	QString aux;
 	double val=0;

 	for(int r=0;r<table->rowCount();r++)
 	 {
 	 	if(table->item(r,rows_det+2)) val=val+table->item(r,rows_det+2)->text().toDouble();
 	 }
 	aux.sprintf("%.2lf",val);
 	return aux;
 }

QString ARScarico::quote(QString str, bool like)
 {
 	if(like)
 	 {
 	  str.replace('\\',"\\\\\\\\");
 	  str.replace('\'',"\\'");
 	 }
 	else
   {
 	  str.replace('\\',"\\\\");
 	  str.replace('\'',"\\'");
   }
 	return str;
 }
